@@ -88,10 +88,12 @@ class ReverseWrapper(object):
         self.declarations = MemoryCodeSink()
         self.post_return_code = MemoryCodeSink()
         self.body = MemoryCodeSink()
+        self.check_exception_code = MemoryCodeSink()
         self.cleanup_actions = []
         self.pyargv_items = []
         self.pyargv_optional_items = []
         self.pyret_parse_items = [] # list of (format_spec, parameter)
+        self.code_sinks_stack = [self.body]
 
     def set_call_target(self, called_pyobj, method_name=None):
         assert called_pyobj is not None
@@ -122,6 +124,14 @@ class ReverseWrapper(object):
         else:
             self.pyret_parse_items.append((format_specifier, parameter))
 
+
+    def push_code_sink(self, code_sink):
+        self.code_sinks_stack.insert(0, code_sink)
+
+    def pop_code_sink(self):
+        return self.code_sinks_stack.pop(0)
+
+
     def write_code(self, code,
                    cleanup=None,
                    failure_expression=None,
@@ -152,7 +162,7 @@ class ReverseWrapper(object):
                       parses the python method return value.
         '''
         if code_sink is None:
-            code_sink = self.body
+            code_sink = self.code_sinks_stack[0]
         if code is not None:
             code_sink.writeln(code)
         if failure_expression is not None:
@@ -170,7 +180,13 @@ class ReverseWrapper(object):
                 code_sink.writeln(failure_cleanup)
             for cleanup_action in self.cleanup_actions:
                 code_sink.writeln(cleanup_action)
-            self.return_type.write_error_return()
+
+            self.push_code_sink(code_sink)
+            try:
+                self.return_type.write_error_return()
+            finally:
+                self.pop_code_sink()
+
             code_sink.unindent()
             code_sink.writeln("}")
         if cleanup is not None:
@@ -265,8 +281,10 @@ class ReverseWrapper(object):
         if self.method_name is None:
             self.write_code("py_retval = PyObject_Call(%s, %s);"
                             % (self.called_pyobj, py_args),
-                            cleanup="Py_DECREF(py_retval);",
-                            failure_expression="!py_retval")
+                            cleanup="Py_XDECREF(py_retval);")
+            self.check_exception_code.flush_to(self.body)
+            self.write_code(None, failure_expression="!py_retval")
+
         else:
             self.add_declaration("PyObject *py_method;")
             self.write_code("py_method = PyObject_GetAttrString(%s, \"%s\");"
@@ -275,8 +293,9 @@ class ReverseWrapper(object):
                             failure_expression="!py_method")
             self.write_code("py_retval = PyObject_CallObject(py_method, %s);"
                             % (py_args,),
-                            cleanup="Py_DECREF(py_retval);",
-                            failure_expression="!py_retval")
+                            cleanup="Py_XDECREF(py_retval);")
+            self.check_exception_code.flush_to(self.body)
+            self.write_code(None, failure_expression="!py_retval")
 
         ## -- Handle the return value --
 
@@ -784,6 +803,17 @@ argtypes.matcher.register_reverse("GdkRectangle*", GdkRectanglePtrParam)
 argtypes.matcher.register_reverse('GtkAllocation*', GdkRectanglePtrParam)
 
 
+class GErrorParam(Parameter):
+    def get_c_type(self):
+        return self.props.get('c_type').replace('**', ' **')
+    def convert_c2py(self):
+        self.wrapper.write_code(code=None,
+            failure_expression=("pyg_gerror_exception_check(%s)" % self.name),
+                                code_sink=self.wrapper.check_exception_code)
+
+argtypes.matcher.register_reverse('GError**', GErrorParam)
+
+
 class PyGObjectMethodParam(Parameter):
     def __init__(self, wrapper, name, method_name, **props):
         Parameter.__init__(self, wrapper, name, **props)
