@@ -50,8 +50,11 @@ else:
 import gdk
 
 if ltihooks:
-    ltihooks.uninstall()
-    del ltihooks
+    try:
+        ltihooks.uninstall()
+        del ltihooks
+    except:
+        pass
 
 from gtk._lazyutils import LazyNamespace, LazyModule
 from gtk.deprecation import _Deprecated, _DeprecatedConstant
@@ -71,7 +74,7 @@ def _init():
         # init_check calls PySys_SetArgv which calls sys.path.insert(0, ''),
         # which causes problems for pychecker, restore it if modified.
         if sys.path != sys_path:
-            sys.path = sys_path
+            sys.path[:] = sys_path
 
     # install the default log handlers
     _gtk.add_log_handlers()
