@@ -22,10 +22,10 @@
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
+
 #include <Python.h>
-/* include this first, before NO_IMPORT_PYGOBJECT is defined */
 #include <pygobject.h>
-#include <pygtk.h>
+#include "pygtk.h"
 #include <glade/glade.h>
 
 void pylibglade_register_classes(PyObject *d);
